/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 * 
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation. Portions created by Netscape are
 * Copyright (C) 1998-1999 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributors:
 *  Stephen Bounds <guruj@mbox.com.au>
 *  Chip Clark <chipc@netscape.com>
 *  Seth Spitzer <sspitzer@netscape.com>
 */

/* NB:
 * requires "chrome://venkman/content/tree-utils.js" preloaded in XUL
 */

/* ------ Custom View definition ------ */

var prefsTree = new XULTreeView();

function PrefsRecord (pref, lock, type, value, hint) 
{
    this.setColumnPropertyName ("prefCol", "pref");
    this.setColumnPropertyName ("lockCol",  "lock");    
    this.setColumnPropertyName ("typeCol",  "type");    
    this.setColumnPropertyName ("valueCol", "value");    
    this.setColumnPropertyName ("hintCol", "hint");    
    this.pref = pref;
    this.lock = lock;
    this.type = type;
    this.value = value;
    this.hint = hint;
}

PrefsRecord.prototype.getCellProperties = 
    function prf_cellprops(index, col, prop) {
        var row = this.childData.locateChildByVisualRow(index);
        if (row._colValues)
           prop.AppendElement(gLockAtoms[row._colValues["lockCol"]]);
        else
           return null;
    };

PrefsRecord.prototype = new XULTreeViewRecord(prefsTree.share);

/* ----- Data generation variables/constants ----- */

const kDefault = 0;
const kUserSet = 1;
const kLocked = 2;

const kStrType = 0;
const kIntType = 1;
const kBoolType = 2;

// XXX get these from a string bundle - ie. a RDF file
var gLockStrs = ["default", "user set","locked"]; 
var gTypeStrs = ["string", "int", "bool"];

const nsIAtomService = Components.interfaces.nsIAtomService;
const nsAtomService_CONTRACTID = "@mozilla.org/atom-service;1";

var atomService = Components.classes[nsAtomService_CONTRACTID].getService(nsIAtomService);
var gLockAtoms = [atomService.getAtom("default"), atomService.getAtom("user"), atomService.getAtom("locked")];

/* ----- Event Trigger Functions ----- */

function onPrefLoad()
{
    document.title = "Preferential - Advanced Preferences Configuration";

    var prefCount = {value:0};
    var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var prefBranch = prefService.getBranch(null);
    var prefArray = prefBranch.getChildList("" , prefCount);
    var prefName, prefType, prefValue, prefIndex, prefLockState;

    var i = 0;   // Counter for iterating through preferences
    var x = 0;   // This is an internal counter for iterating through
                 // headings 

    // these arrays contain preference hierarchy information to determine
    // when new headings are required to be started/finished
    var thisHierarchy = new Array();
    var lastHierarchy = new Array();

    // max number of splits for preference hierarchies
    const splitLimit = 4;

    // contains the tree items currently being worked on in the hierarchy
    var level = new Array(splitLimit);
    
    // Once array is sorted, we can create hierarchy trees based upon
    // similarities and finding '.'s
    prefArray.sort();

    for (i = 0; i < prefCount.value; i++) 
    {
        // DEBUGGING flag
        var _DEBUG = false;

	  // avoid displaying "private" preferences
        if((prefArray[i].indexOf("capability", 0) + 1) > 0) continue;

        // generate tree row values
        if (prefBranch.prefIsLocked(prefArray[i]))
          prefLockState = kLocked;
        else if(prefBranch.prefHasUserValue(prefArray[i]))
          prefLockState = kUserSet;
        else
          prefLockState = kDefault;

        const nsIPrefBranch = Components.interfaces.nsIPrefBranch;

        switch (prefBranch.getPrefType(prefArray[i])) {
            case nsIPrefBranch.PREF_INT:
                prefType = kIntType;
                // convert to a string
                prefValue = "" + prefBranch.getIntPref(prefArray[i]);
                    break;
            case nsIPrefBranch.PREF_BOOL:
                prefType = kBoolType;
                // convert to a string
                if (prefBranch.getBoolPref(prefArray[i]))
                  prefValue = "true";
                else
                  prefValue = "false";
                    break;
            case nsIPrefBranch.PREF_STRING:
              prefType = kStrType;
              prefValue = htmlEscape(prefBranch.getCharPref(prefArray[i]));
                  break;
            default:
              alert('Error with element ' + prefArray[i]);
        }

        // copy old hierarchy
        delete lastHierarchy;
        for (x=0; x<thisHierarchy.length; x++) {
            lastHierarchy[x] = thisHierarchy[x];
        }

        // generate new hierarchy        
        delete thisHierarchy;
        thisHierarchy = htmlEscape(prefArray[i]).split('.',splitLimit);
        for (x=1; x<thisHierarchy.length; x++) {
            thisHierarchy[x] = thisHierarchy[x-1] + '.' + thisHierarchy[x];
        }

        if (_DEBUG) alert('this:\n' + 
                 + thisHierarchy[0] + '\n' + thisHierarchy[1] + '\n'
                 + thisHierarchy[2] + '\n' + thisHierarchy[3] + '\n'
     + 'last:\n' + lastHierarchy[0] + '\n' + lastHierarchy[1] + '\n\n'
                 + lastHierarchy[2] + '\n' + lastHierarchy[3] + '\n');
        
        // insert new headings as required
        for (x=thisHierarchy.length-1; x >= 0; x--) {
            // unset values for any non-leaf entries
            if (x < thisHierarchy.length-1) {
                prefLockState=""; prefType=""; prefValue="";
            }

            // FURTHER UP THE TREE
            if (thisHierarchy[x] != lastHierarchy[x] || 
                lastHierarchy[x] == undefined) {
                if (_DEBUG) alert('creating record: (overwrite)');
                // TODO: need to generate 'hint' if found in RDF datastore
                level[x] = new PrefsRecord(thisHierarchy[x], 
                    prefLockState, prefType, prefValue, "");
                nextAppendChild = true;
            }

            // append children (if any) to this node
            if (x < (thisHierarchy.length-1)) {
                if (thisHierarchy[x+1] != lastHierarchy[x+1]) {
                    if (_DEBUG) alert(i + ': appending: ' + thisHierarchy[x+1] + ' to ' + thisHierarchy[x]);            
                    if (level[x].childData == undefined)
                        level[x].reserveChildren();
                    level[x].appendChild(level[x+1]);
                }
            }

            // SPECIAL HANDLING FOR ROOT ENTRIES (ie. x=0)
            if (x==0) {
                if (_DEBUG) alert(i + ': handle root');
                // append to root of tree each time base level changes
                if (thisHierarchy[x] != lastHierarchy[x] || 
                   lastHierarchy[x] == undefined) {
                    if (_DEBUG) alert(i + ': append to root: ' + thisHierarchy[x]);
                    prefsTree.childData.appendChild(level[x]);
                }
            }
            if (_DEBUG) alert('done this loop');
        }
    }

    var gTree = document.getElementById("prefsTree");
    gTree.treeBoxObject.view = prefsTree;
}

function PrefNewClick()
{
    var row = prefsTree.childData.locateChildByVisualRow(prefsTree.selectedIndex);
    window.openDialog("chrome://preferential/content/preferential-new.xul",
                      "newPrefDialog","chrome",row.pref,row.lock,row.type,row.value);
}

function PrefEditClick()
{
    var row = prefsTree.childData.locateChildByVisualRow(prefsTree.selectedIndex);
    
    // boolean preferences get an enumerated window
    if (row.type == 2) {
        var enumArray = Array(2);
        enumArray[0] = {label:"true", value:-1}; enumArray[1] = {label:"false", value:0};
        window.openDialog("chrome://preferential/content/preferential-edit-enum.xul",
                      "editPrefDialog","chrome",row.pref,row.lock,row.type,row.value,enumArray);
    }
    // TODO: if RDF enum list found, open enumerated window
    else {
        window.openDialog("chrome://preferential/content/preferential-edit.xul",
                              "editPrefDialog","chrome",row.pref,row.lock,row.type,row.value);
    }

}

function PrefDeleteClick()
{
    var row = prefsTree.childData.locateChildByVisualRow(prefsTree.selectedIndex);
    window.openDialog("chrome://preferential/content/preferential-delete.xul",
                      "deletePrefDialog","chrome",row.pref,row.lock,row.type,row.value);
}

/* --------- UTILITY FUNCTIONS --------- */

function htmlEscape(s) 
{ 
    s = s.replace(/\&/g, "&amp;"); 
    s = s.replace(/\>/g, "&gt;"); 
    s = s.replace(/\</g, "&lt;"); 
    return s; 
}

